#!/usr/bin/perl

# Copyright (C) 2004 Billy Biggs <vektor@dumbterm.net>.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# 0.1 - Fri Feb 20 21:34:48 AST 2004
#   - Initial release, needs lots of work.

use strict;
use Time::HiRes qw( usleep );
use POSIX "sys_wait_h";

my $control = "/mp3/control/";
my $mp3player = "/usr/bin/mpg123";

my @names;
my $curtrk;
my $curdir;
my $curpid;

# Play the given mp3 file, returns the PID of the process.
sub playMp3 {
    my $filename = shift;
    my $pid;
    my @args;

    if($pid = fork) {
        return $pid;
    } else {
        $args[0] = $mp3player;
        $args[1] = $filename;
        exec(@args);
    }
}

# This can fail and stop the daemon although that's
# really stupid.
sub getNames {
    my $dirname = shift;
    my $dir;
    my @files;
    my @names;
    my $cur;
    my $i;

    opendir($dir, $dirname) or die "Can't open directory: $!";
    @files = readdir($dir);
    $cur = 0;
    foreach $i (@files) {
        $_ = $i;
        if(/\.mp3$/) {
            @names[$cur++] = $i;
        }
    }
    closedir($dir);
    @names = sort(@names);
    return @names;
}

sub nowPlaying {
    my $dirname = shift;
    my $trackname = shift;
    my $file;
    my $album;
    my $track;

    $album = $dirname;
    $album =~ s/\/$//;
    $album =~ s/.*\///;

    open($file, ">$control" . "nowplaying-album");
    print($file "$album\n");
    close($file);

    $track = $trackname;
    $track =~ s/\.mp3$//;

    open($file, ">$control" . "nowplaying-track");
    print($file "$track\n");
    close($file);
}

sub trackListing {
    my $names = shift;
    my $curplaying = shift;
    my $file;
    my $i;
    my $temp;
    my $cur = 0;

    open($file, ">$control" . "nowplaying-tracklist");
    foreach $i (@$names) {
        $temp = $i;
        $temp =~ s/\.mp3$//;
        if( $cur == $curplaying ) {
            print($file "<tr><td class=\"trackplaying\"><a href=\"?play=please&trackid=$cur\">$temp</a>\n");
        } else {
            print($file "<tr><td class=\"track\"><a href=\"?play=please&trackid=$cur\">$temp</a>\n");
        }
	$cur++;
    }
    close($file);
}

$curdir = $ARGV[0];
@names = getNames($curdir);
$curtrk = 0;
$curpid = playMp3($curdir . "/" . $names[$curtrk]);
nowPlaying($curdir, $names[$curtrk]);
trackListing(\@names, $curtrk);

for(;;) {
    if(-e $control . "skip") {
        if($curpid >= 0) {
            kill('INT',$curpid);
            waitpid($curpid, 0);
            $curpid = -1;
            $curtrk++;
            if($curtrk < scalar @names) {
                $curpid = playMp3($curdir . "/" . $names[$curtrk]);
            } else {
                $curpid = -1;
                $curtrk = 0;
            }
        }
        nowPlaying($curdir, $names[$curtrk]);
        trackListing(\@names, $curtrk);
        unlink($control . "skip");
    }
    if(-e $control . "prev") {
        if($curpid >= 0) {
            kill('INT',$curpid);
            waitpid($curpid, 0);
            $curpid = -1;
            $curtrk--;
            if($curtrk < 0) {
                $curtrk = 0;
            }
            $curpid = playMp3($curdir . "/" . $names[$curtrk]);
        }
        nowPlaying($curdir, $names[$curtrk]);
        trackListing(\@names, $curtrk);
        unlink($control . "prev");
    }
    if(-e $control . "loud") {
        system("aumix -w 80");
        unlink($control . "loud");
    }
    if(-e $control . "mid") {
        system("aumix -w 60");
        unlink($control . "mid");
    }
    if(-e $control . "quiet") {
        system("aumix -w 40");
        unlink($control . "quiet");
    }
    if(-e $control . "stop") {
        if($curpid >= 0) {
            kill('INT',$curpid);
            waitpid($curpid, 0);
            $curpid = -1;
        }
        nowPlaying($curdir, $names[$curtrk]);
        trackListing(\@names, $curtrk);
        unlink($control . "stop");
    }
    if(-e $control . "rewind") {
        if($curpid >= 0) {
            kill('INT',$curpid);
            waitpid($curpid, 0);
            $curpid = -1;
            $curtrk = 0;
        }
        nowPlaying($curdir, $names[$curtrk]);
        trackListing(\@names, $curtrk);
        unlink($control . "rewind");
    }
    if(-e $control . "play") {
        if($curpid < 0 ) {
            $curpid = playMp3($curdir . "/" . $names[$curtrk]);
        }
        nowPlaying($curdir, $names[$curtrk]);
        trackListing(\@names, $curtrk);
        unlink($control . "play");
    }
    if(-e $control . "loadtrack") {
        my $file;

        if($curpid >= 0) {
            kill('INT', $curpid);
            waitpid($curpid, 0);
            $curpid = -1;
        }

        open($file, $control . "track");
        $curtrk = <$file>;
	$curtrk =~ s/\n$//;
        close($file);
        if($curtrk < scalar @names) {
            $curpid = playMp3($curdir . "/" . $names[$curtrk]);
        } else {
            $curpid = -1;
            $curtrk = 0;
        }

        nowPlaying($curdir, $names[$curtrk]);
        trackListing(\@names, $curtrk);
        unlink($control . "track");
        unlink($control . "loadtrack");
    }
    if(-e $control . "load") {
        my $file;

        if($curpid >= 0) {
            kill('INT', $curpid);
            waitpid($curpid, 0);
            $curpid = -1;
        }
        open($file, $control . "disc");
        $curdir = <$file>;
	$curdir =~ s/\n$//;
        close($file);
        @names = getNames($curdir);
        trackListing(\@names, $curtrk);
        $curtrk = 0;
        if(scalar @names > 0) {
            $curpid = playMp3($curdir . "/" . $names[$curtrk]);
        }

        nowPlaying($curdir, $names[$curtrk]);
        trackListing(\@names, $curtrk);
        unlink($control . "disc");
        unlink($control . "load");
    }
    if(-e $control . "quit") {
        if($curpid >= 0) {
            kill('INT', $curpid);
            waitpid($curpid, 0);
            $curpid = -1;
        }
        unlink($control . "quit");
        exit(0);
    }
    waitpid($curpid,WNOHANG);
    if($? >= 0 ) {
        $curtrk++;
        if($curtrk < scalar @names) {
            $curpid = playMp3($curdir . "/" . $names[$curtrk]);
        } else {
            $curpid = -1;
            $curtrk = 0;
        }
        nowPlaying($curdir, $names[$curtrk]);
        trackListing(\@names, $curtrk);
    }
    usleep(50000);
}

